Plotly.setPlotConfig({
  topojsonURL: "Library/plotly/topojson/"
});

d3.csv("Data/AlcoholUseDisorders.csv", function (err, rows) {

  function filter_and_unpack(rows, key, year) {
    return rows.filter(row => row['Year'] == year).map(row => row[key])
  }

  var frames = []
  var slider_steps = []

  var n = 27;
  var currentYear = 1990;
  for (var i = 0; i <= n; i++) {
    var z = filter_and_unpack(rows, 'DeathRateRounded', currentYear)
    var locations = filter_and_unpack(rows, 'Code', currentYear)
    var countries = filter_and_unpack(rows, 'Country', currentYear)
    frames[i] = {
      data: [{
        z: z,
        locations: locations,
        countries: countries,
        text: countries
      }], name: currentYear
    }
    slider_steps.push({
      label: currentYear.toString(),
      method: "animate",
      args: [[currentYear], {
        mode: "immediate",
        transition: { duration: 300 },
        frame: { duration: 300 }
      }
      ]
    })
    currentYear = currentYear + 1
  }

  var data = [{
    type: 'choropleth',
    locationmode: 'world',
    locations: frames[0].data[0].locations,
    z: frames[0].data[0].z,
    text: frames[0].data[0].countries,
    zauto: false,
    zmin: 0,
    zmax: rows.reduce((prev, curr) => Math.max(prev, curr.DeathRateRounded), 0),
    colorscale: [
      [0, '#eee'], [0.25, '#5277D6'], [1, '#650023']],
    autocolorscale: false,
    hovertemplate: "<b>%{text}</b><br>" + "Rate: %{z}" + "<extra></extra>"
  }];

  var layout = {
    dragmode: false,
    title: {
      text: 'Death rates from alcohol use disease<br><span style="font-size: 14px">Age-standardized death rates, measured per 100,000 individuals.',
      font: {
        size: 20,
        family: 'Sans-Serif'
      }
    },
    geo: {
      scope: 'world',
      countrycolor: 'rgb(255, 255, 255)',
      showland: true,
      landcolor: '#fff',
      showlakes: true,
      lakecolor: 'rgb(255, 255, 255)',
      subunitcolor: 'rgb(255, 255, 255)',
      lonaxis: {},
      lataxis: {}
    },
    updatemenus: [{
      x: 0.1,
      y: 0,
      yanchor: "top",
      xanchor: "right",
      showactive: false,
      direction: "left",
      type: "buttons",
      pad: { "t": 87, "r": 10 },
      buttons: [{
        method: "animate",
        args: [null, {
          fromcurrent: true,
          transition: {
            duration: 200,
          },
          frame: {
            duration: 750
          }
        }],
        label: "<span style=\"font-family: 'Font Awesome 5 Free'; font-weight: 900;\">\uf04b</span>"
      }]
    }],
    sliders: [{
      active: 0,
      steps: slider_steps,
      x: 0.1,
      len: 0.9,
      xanchor: "left",
      y: 0,
      yanchor: "top",
      pad: { t: 50, b: 10 },
      currentvalue: {
        visible: true,
        prefix: "Year: ",
        xanchor: "right",
        font: {
          size: 20,
          color: "#666",
          family: 'Sans-Serif'
        }
      },
      transition: {
        duration: 300,
        easing: "cubic-in-out"
      }
    }]
  };

  var config = {
    displayModeBar: false,
    scrollZoom: false,
    responsive: true
  }

  Plotly.newPlot("disorder", data, layout, config).then(function (e) {
    Plotly.addFrames("disorder", frames);

    e.on('plotly_click', function (getData) {
      var countryrows = rows.filter(e => e.Code == getData.points[0].location);

      var data = [{
        x: countryrows.map(e => e.Year),
        y: countryrows.map(e => e.DeathRateRounded),
        mode: 'lines',
        hovertemplate: "%{y}" + "<extra></extra>"
      }];

      var layout = {
        dragmode: false,
        title: {
          text: `Death rate from alcohol use disease ${countryrows[0].Country}<br><span style="font-size: 14px">Age-standardized death rates, measured per 100,000 individuals.`,
          font: {
            size: 20,
            family: 'Sans-Serif'
          }
        },
        xaxis: {
          title: {
            text: 'Year',
            font: {
              size: 18,
              color: '#7f7f7f',
              family: 'Sans-Serif'
            }
          },
          tickfont: {
            size: 14,
          }
        },
        yaxis: {
          title: {
            text: 'Death rate',
            font: {
              size: 18,
              color: '#7f7f7f',
              family: 'Sans-Serif'
            }
          },
          tickfont: {
            size: 14,
            family: 'Sans-Serif'
          }
        }
      }

      var config = {
        displayModeBar: false,
        scrollZoom: false,
        responsive: true
      }

      Plotly.react('lineChart', data, layout, config);

      Plotly.animate('disorder', [null],
        {
          mode: "immediate",
          transition: {
            duration: 0
          },
          frame: {
            duration: 0
          }
        });

      document.getElementById('overlay').classList.remove("hidden");
    });
  });
})
