d3.csv("/Data/Scatterplot.csv", function (err, rows) {
    function unpack(rows, key) {
        return rows.map(function (row) { return row[key]; });
    }


    var regionColors = {
        "Eastern Mediterranean": "#F3722C",
        "Europe": "#F8961E",
        "Africa": "#F9C74F",
        "Americas": "#90BE6D",
        "Western Pacific": "#43AA8B",
        "South-East Asia": "#577590",
    }

    var hale = unpack(rows, "HALE at birth").map(function (row) {
        return (row !== '') ? Number(row) : row;
    });
    var onlyNumbers = hale.filter(function (row) { return row !== '' });
    var minValue = Math.min(...onlyNumbers);
    var maxValue = Math.max(...onlyNumbers);
    var valueDiff = maxValue - minValue;
    var haleNormalised = hale.map(function (row) {
        return (row !== '') ? (row - minValue) / valueDiff : row;
    });

    var countries = unpack(rows, "Country");
    var expectancies = unpack(rows, "HALE at birth (rounded)");

    var data = [
        {
            x: unpack(rows, "Alcohol consumption"),
            y: unpack(rows, "Happiness 2019"),
            text: countries.map(function (row, index) { return { country: row, expectancy: expectancies[index] } }),
            mode: "markers",
            marker: {
                size: haleNormalised.map(function (row) { return 5 + row * 15 }),
                color: unpack(rows, "WHO region").map(function (row) { return regionColors[row] }),
            },
            type: "scatter",
            indicator: {
                visible: false,
            },
            hovertemplate:
                "<b>%{text.country}</b>" +
                "<br>Happiness index: %{y}" +
                "<br>Alcohol consumption: %{x} L" +
                "<br>Life expectancy: %{text.expectancy} years" +
                "<extra></extra>",
        }
    ];

    var layout = {
        title: "Happiness vs alcohol consumption",
        hovermode: "closest",
        xaxis: {
            fixedrange: true,
            title: "Pure alcohol consumption (in litres)",

        },
        yaxis: {
            fixedrange: true,
            title: "Ranking of happiness",
        },
    };

    var config = {
        displayModeBar: false,
        scrollZoom: false,
        responsive: true,
    }

    Plotly.newPlot("happiness", data, layout, config);
})
